Unit uDynamicLoad;

{$mode objFPC}{$H+}

{$DEFINE LOAD_DYNAMIC}

Interface

Uses
  Classes,
  SysUtils;

Const
  TESSERACT_LIBRARY = 'tesseract41.dll';
  LAME_LIBRARY = 'libmp3lame.dll';
  INVALID_HANDLE_VALUE = THANDLE(-1);


{$IFDEF LOAD_DYNAMIC}
Type
  TFnTessVersion    = Function: PChar; Cdecl;
  TFnGetLameVersion = Function: PChar; Cdecl;
  TFnGetLameUrl     = Function: PChar; Cdecl;

Var
  TessVersion       : TFnTessVersion;
  get_lame_version  : TFnGetLameVersion;
  get_lame_Url      : TFnGetLameUrl;
  {$ELSE}
    Function TessVersion       : pChar; cdecl; external TESSERACT_LIBRARY name 'TessVersion';
    function get_lame_version  : pChar; cdecl; external LAME_LIBRARY name 'get_lame_version';
    function get_lame_url      : pChar; cdecl; external LAME_LIBRARY name 'get_lame_url';
  {$ENDIF}

Implementation


{$IFDEF LOAD_DYNAMIC}
Var
  hTesseract : TLibHandle;
  hLame      : TLibHandle;


Function IsValidHandle(Const AHandle: THandle): Boolean;
Begin
  Result := (AHandle <> 0) And (AHandle <> INVALID_HANDLE_VALUE);
End;

Procedure LoadTesseract;
Begin
  TessVersion := Nil;
  hTesseract := LoadLibrary(TESSERACT_LIBRARY);
  If IsValidHandle(hTesseract) Then
  Begin
    TessVersion := TFnTessVersion(GetProcAddress(hTesseract, 'TessVersion'));
    // Pointer(TessVersion) := GetProcedureAddress(hTesseract, 'TessVersion');
  End;
End;

Procedure UnLoadTesseract;
Begin
  If IsValidHandle(hTesseract) Then
  Begin
    UnLoadLibrary(hTesseract);
  End;
End;

Procedure LoadLame;
Begin
  get_lame_version := Nil;
  get_lame_url := Nil;
  hLame := LoadLibrary(LAME_LIBRARY);
  If IsValidHandle(hLame) Then
  Begin
    get_lame_version := TFnGetLameVersion(GetProcedureAddress(hLame, 'get_lame_version'));
    get_lame_url := TFnGetLameUrl(GetProcedureAddress(hLame, 'get_lame_url'));
  End;
End;

Procedure UnLoadLame;
Begin
  If IsValidHandle(hLame) Then
  Begin
    UnLoadLibrary(hLame);
  End;
End;



Initialization
  LoadTesseract;
  LoadLame;

Finalization
  UnLoadLame;
  UnLoadTesseract;

{$ENDIF}

End.
